var UrlEngine = function($location, $rootScope, eventAggregator, ScreenService) {
	var changedByOpen = false;
	var changedByUrl = false;
	var changedByInit = false;

	this.init = function(project) {
		var containerName = getContainerName($location);
		var shouldBackToMainWindowOnReload = project.shouldBackToMainWindowOnReload;

		if(project.shouldBackToMainWindowOnReloadExceptions && project.shouldBackToMainWindowOnReloadExceptions.indexOf(containerName) !== -1){
			shouldBackToMainWindowOnReload = !project.shouldBackToMainWindowOnReload;
		}

		if (!containerName || (shouldBackToMainWindowOnReload && !inDevToolsMode())) {
			containerName = project.mainWindow;
		}

		changedByInit = true;
		ScreenService.openWindow(containerName)
			.then(function () {
				//during the first initialization, set changedByOpen to false because of cases when the user enters a container url directly
				changedByOpen = false;
			});

		eventAggregator.subscribe(EventAggregator.ON_CONTAINER_OPEN, function(openEvent) {
			if(!changedByUrl) {
				changedByOpen = true;
				var currentQueryString = "";
				if (changedByInit && urlHasQueryString($location.url())) {
					currentQueryString = "?" + getQueryStringFromUrl($location.url());
					changedByInit = false;
				}
				$location.url("/" + Util.getContainerJsonName(openEvent.container) + currentQueryString);
			}
			changedByUrl = false;
		});

		$rootScope.$on("$locationChangeStart", function(event, next, current) {
			eventAggregator.publish("locationChangeStart", {
				event: event,
				next: next,
				current: current
			});
		});
		$rootScope.$on("$locationChangeSuccess", function(event, newUrl, oldUrl) {
			eventAggregator.publish("locationChangeSuccess", {
				event: event,
				newUrl: newUrl,
				oldUrl: oldUrl
			});
			if(!changedByOpen){
				changedByUrl = true;
				var containerName = getContainerName($location);
				ScreenService.openWindow(containerName);
			} else {
				changedByOpen = false;
			}
		});
	};

	function getContainerName($location){
		var hash = $location.hash() ? "#" + $location.hash() : "";
		var path = $location.path() + hash;
		return path.replace("/","");
	}

	function inDevToolsMode() {
		return $location.search().devtools;
	}

	function urlHasQueryString(url) {
		return url.split("?").length === 2;
	}

	function getQueryStringFromUrl(url) {
		return url.split("?").pop();
	}
};